/*
   Copyright (C) 2012-2020 Laurent Montel <montel@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef IMPORTWIZARDUTIL_H
#define IMPORTWIZARDUTIL_H
#include <QUrl>
#include <QColor>
#include <QString>
#include "libimportwizard_export.h"

struct LIBIMPORTWIZARD_EXPORT ldapStruct {
    ldapStruct()
        : maxHint(-1)
        , port(-1)
        , limit(-1)
        , timeout(-1)
        , useSSL(false)
        , useTLS(false)
    {
    }

    QUrl ldapUrl;
    QString dn;
    QString saslMech;
    QString fileName;
    QString description;
    QString password;
    int maxHint;
    int port;
    int limit;
    int timeout;
    bool useSSL;
    bool useTLS;
};

struct tagStruct {
    QString name;
    QColor color;
};
Q_DECLARE_TYPEINFO(tagStruct, Q_MOVABLE_TYPE);

namespace ImportWizardUtil {
enum ResourceType {
    Imap,
    Pop3,
    Ldap
};

LIBIMPORTWIZARD_EXPORT void mergeLdap(const ldapStruct &ldap);
LIBIMPORTWIZARD_EXPORT void addAkonadiTag(const QVector<tagStruct> &tagList);
void storeInKWallet(const QString &name, ImportWizardUtil::ResourceType type, const QString &password);
}

#endif // IMPORTWIZARDUTIL_H
